# coding: utf-8

"""
    Kestra EE

    All API operations, except for Superadmin-only endpoints, require a tenant identifier in the HTTP path.<br/> Endpoints designated as Superadmin-only are not tenant-scoped.

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501

import warnings
from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Any, Dict, List, Optional, Tuple, Union
from typing_extensions import Annotated

from pydantic import StrictInt, StrictStr
from typing import Any, Optional
from kestrapy.models.patch_request import PatchRequest
from kestrapy.models.scim_resource import ScimResource
from kestrapy.models.scim_user import ScimUser
from kestrapy.models.search_request import SearchRequest

from kestrapy.api_client import ApiClient, RequestSerialized
from kestrapy.api_response import ApiResponse
from kestrapy.rest import RESTResponseType


class SCIMUsersApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client


    @validate_call
    def create_scim_resource_by_id_users(
        self,
        integration: StrictStr,
        tenant: StrictStr,
        scim_user: ScimUser,
        attributes: Optional[StrictStr] = None,
        excluded_attributes: Optional[StrictStr] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ScimResource:
        """create_scim_resource_by_id_users

        Create

        :param integration: (required)
        :type integration: str
        :param tenant: (required)
        :type tenant: str
        :param scim_user: (required)
        :type scim_user: ScimUser
        :param attributes:
        :type attributes: str
        :param excluded_attributes:
        :type excluded_attributes: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._create_scim_resource_by_id_users_serialize(
            integration=integration,
            tenant=tenant,
            scim_user=scim_user,
            attributes=attributes,
            excluded_attributes=excluded_attributes,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '201': "ScimResource",
            '400': None,
            '409': None,
            '500': None,
            '501': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def create_scim_resource_by_id_users_with_http_info(
        self,
        integration: StrictStr,
        tenant: StrictStr,
        scim_user: ScimUser,
        attributes: Optional[StrictStr] = None,
        excluded_attributes: Optional[StrictStr] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[ScimResource]:
        """create_scim_resource_by_id_users

        Create

        :param integration: (required)
        :type integration: str
        :param tenant: (required)
        :type tenant: str
        :param scim_user: (required)
        :type scim_user: ScimUser
        :param attributes:
        :type attributes: str
        :param excluded_attributes:
        :type excluded_attributes: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._create_scim_resource_by_id_users_serialize(
            integration=integration,
            tenant=tenant,
            scim_user=scim_user,
            attributes=attributes,
            excluded_attributes=excluded_attributes,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '201': "ScimResource",
            '400': None,
            '409': None,
            '500': None,
            '501': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def create_scim_resource_by_id_users_without_preload_content(
        self,
        integration: StrictStr,
        tenant: StrictStr,
        scim_user: ScimUser,
        attributes: Optional[StrictStr] = None,
        excluded_attributes: Optional[StrictStr] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """create_scim_resource_by_id_users

        Create

        :param integration: (required)
        :type integration: str
        :param tenant: (required)
        :type tenant: str
        :param scim_user: (required)
        :type scim_user: ScimUser
        :param attributes:
        :type attributes: str
        :param excluded_attributes:
        :type excluded_attributes: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._create_scim_resource_by_id_users_serialize(
            integration=integration,
            tenant=tenant,
            scim_user=scim_user,
            attributes=attributes,
            excluded_attributes=excluded_attributes,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '201': "ScimResource",
            '400': None,
            '409': None,
            '500': None,
            '501': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _create_scim_resource_by_id_users_serialize(
        self,
        integration,
        tenant,
        scim_user,
        attributes,
        excluded_attributes,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if integration is not None:
            _path_params['integration'] = integration
        if tenant is not None:
            _path_params['tenant'] = tenant
        # process the query parameters
        if attributes is not None:
            
            _query_params.append(('attributes', attributes))
            
        if excluded_attributes is not None:
            
            _query_params.append(('excludedAttributes', excluded_attributes))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if scim_user is not None:
            _body_params = scim_user


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/scim+json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/scim+json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/api/v1/{tenant}/integrations/{integration}/scim/v2/Users',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def delete_users(
        self,
        id: StrictStr,
        integration: StrictStr,
        tenant: StrictStr,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> None:
        """delete_users

        Delete from the backing store

        :param id: (required)
        :type id: str
        :param integration: (required)
        :type integration: str
        :param tenant: (required)
        :type tenant: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._delete_users_serialize(
            id=id,
            integration=integration,
            tenant=tenant,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
            '400': None,
            '404': None,
            '500': None,
            '501': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def delete_users_with_http_info(
        self,
        id: StrictStr,
        integration: StrictStr,
        tenant: StrictStr,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[None]:
        """delete_users

        Delete from the backing store

        :param id: (required)
        :type id: str
        :param integration: (required)
        :type integration: str
        :param tenant: (required)
        :type tenant: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._delete_users_serialize(
            id=id,
            integration=integration,
            tenant=tenant,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
            '400': None,
            '404': None,
            '500': None,
            '501': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def delete_users_without_preload_content(
        self,
        id: StrictStr,
        integration: StrictStr,
        tenant: StrictStr,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """delete_users

        Delete from the backing store

        :param id: (required)
        :type id: str
        :param integration: (required)
        :type integration: str
        :param tenant: (required)
        :type tenant: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._delete_users_serialize(
            id=id,
            integration=integration,
            tenant=tenant,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
            '400': None,
            '404': None,
            '500': None,
            '501': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _delete_users_serialize(
        self,
        id,
        integration,
        tenant,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if id is not None:
            _path_params['id'] = id
        if integration is not None:
            _path_params['integration'] = integration
        if tenant is not None:
            _path_params['tenant'] = tenant
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter




        # authentication setting
        _auth_settings: List[str] = [
        ]

        return self.api_client.param_serialize(
            method='DELETE',
            resource_path='/api/v1/{tenant}/integrations/{integration}/scim/v2/Users/{id}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def find_users(
        self,
        integration: StrictStr,
        tenant: StrictStr,
        search_request: SearchRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ScimResource:
        """find_users

        Search

        :param integration: (required)
        :type integration: str
        :param tenant: (required)
        :type tenant: str
        :param search_request: (required)
        :type search_request: SearchRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._find_users_serialize(
            integration=integration,
            tenant=tenant,
            search_request=search_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '400': None,
            '500': None,
            '501': None,
            '200': "ScimResource",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def find_users_with_http_info(
        self,
        integration: StrictStr,
        tenant: StrictStr,
        search_request: SearchRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[ScimResource]:
        """find_users

        Search

        :param integration: (required)
        :type integration: str
        :param tenant: (required)
        :type tenant: str
        :param search_request: (required)
        :type search_request: SearchRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._find_users_serialize(
            integration=integration,
            tenant=tenant,
            search_request=search_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '400': None,
            '500': None,
            '501': None,
            '200': "ScimResource",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def find_users_without_preload_content(
        self,
        integration: StrictStr,
        tenant: StrictStr,
        search_request: SearchRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """find_users

        Search

        :param integration: (required)
        :type integration: str
        :param tenant: (required)
        :type tenant: str
        :param search_request: (required)
        :type search_request: SearchRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._find_users_serialize(
            integration=integration,
            tenant=tenant,
            search_request=search_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '400': None,
            '500': None,
            '501': None,
            '200': "ScimResource",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _find_users_serialize(
        self,
        integration,
        tenant,
        search_request,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if integration is not None:
            _path_params['integration'] = integration
        if tenant is not None:
            _path_params['tenant'] = tenant
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if search_request is not None:
            _body_params = search_request


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/scim+json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/scim+json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/api/v1/{tenant}/integrations/{integration}/scim/v2/Users/.search',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def get_scim_resource_by_id_users(
        self,
        id: StrictStr,
        integration: StrictStr,
        tenant: StrictStr,
        attributes: Optional[StrictStr] = None,
        excluded_attributes: Optional[StrictStr] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ScimResource:
        """get_scim_resource_by_id_users

        Find by id

        :param id: (required)
        :type id: str
        :param integration: (required)
        :type integration: str
        :param tenant: (required)
        :type tenant: str
        :param attributes:
        :type attributes: str
        :param excluded_attributes:
        :type excluded_attributes: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_scim_resource_by_id_users_serialize(
            id=id,
            integration=integration,
            tenant=tenant,
            attributes=attributes,
            excluded_attributes=excluded_attributes,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '400': None,
            '404': None,
            '500': None,
            '501': None,
            '200': "ScimResource",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_scim_resource_by_id_users_with_http_info(
        self,
        id: StrictStr,
        integration: StrictStr,
        tenant: StrictStr,
        attributes: Optional[StrictStr] = None,
        excluded_attributes: Optional[StrictStr] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[ScimResource]:
        """get_scim_resource_by_id_users

        Find by id

        :param id: (required)
        :type id: str
        :param integration: (required)
        :type integration: str
        :param tenant: (required)
        :type tenant: str
        :param attributes:
        :type attributes: str
        :param excluded_attributes:
        :type excluded_attributes: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_scim_resource_by_id_users_serialize(
            id=id,
            integration=integration,
            tenant=tenant,
            attributes=attributes,
            excluded_attributes=excluded_attributes,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '400': None,
            '404': None,
            '500': None,
            '501': None,
            '200': "ScimResource",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_scim_resource_by_id_users_without_preload_content(
        self,
        id: StrictStr,
        integration: StrictStr,
        tenant: StrictStr,
        attributes: Optional[StrictStr] = None,
        excluded_attributes: Optional[StrictStr] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """get_scim_resource_by_id_users

        Find by id

        :param id: (required)
        :type id: str
        :param integration: (required)
        :type integration: str
        :param tenant: (required)
        :type tenant: str
        :param attributes:
        :type attributes: str
        :param excluded_attributes:
        :type excluded_attributes: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_scim_resource_by_id_users_serialize(
            id=id,
            integration=integration,
            tenant=tenant,
            attributes=attributes,
            excluded_attributes=excluded_attributes,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '400': None,
            '404': None,
            '500': None,
            '501': None,
            '200': "ScimResource",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_scim_resource_by_id_users_serialize(
        self,
        id,
        integration,
        tenant,
        attributes,
        excluded_attributes,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if id is not None:
            _path_params['id'] = id
        if integration is not None:
            _path_params['integration'] = integration
        if tenant is not None:
            _path_params['tenant'] = tenant
        # process the query parameters
        if attributes is not None:
            
            _query_params.append(('attributes', attributes))
            
        if excluded_attributes is not None:
            
            _query_params.append(('excludedAttributes', excluded_attributes))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/scim+json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/api/v1/{tenant}/integrations/{integration}/scim/v2/Users/{id}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def patch_users(
        self,
        id: StrictStr,
        integration: StrictStr,
        tenant: StrictStr,
        patch_request: PatchRequest,
        attributes: Optional[StrictStr] = None,
        excluded_attributes: Optional[StrictStr] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> None:
        """patch_users

        Patch a portion of the backing store

        :param id: (required)
        :type id: str
        :param integration: (required)
        :type integration: str
        :param tenant: (required)
        :type tenant: str
        :param patch_request: (required)
        :type patch_request: PatchRequest
        :param attributes:
        :type attributes: str
        :param excluded_attributes:
        :type excluded_attributes: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._patch_users_serialize(
            id=id,
            integration=integration,
            tenant=tenant,
            patch_request=patch_request,
            attributes=attributes,
            excluded_attributes=excluded_attributes,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
            '400': None,
            '404': None,
            '500': None,
            '501': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def patch_users_with_http_info(
        self,
        id: StrictStr,
        integration: StrictStr,
        tenant: StrictStr,
        patch_request: PatchRequest,
        attributes: Optional[StrictStr] = None,
        excluded_attributes: Optional[StrictStr] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[None]:
        """patch_users

        Patch a portion of the backing store

        :param id: (required)
        :type id: str
        :param integration: (required)
        :type integration: str
        :param tenant: (required)
        :type tenant: str
        :param patch_request: (required)
        :type patch_request: PatchRequest
        :param attributes:
        :type attributes: str
        :param excluded_attributes:
        :type excluded_attributes: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._patch_users_serialize(
            id=id,
            integration=integration,
            tenant=tenant,
            patch_request=patch_request,
            attributes=attributes,
            excluded_attributes=excluded_attributes,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
            '400': None,
            '404': None,
            '500': None,
            '501': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def patch_users_without_preload_content(
        self,
        id: StrictStr,
        integration: StrictStr,
        tenant: StrictStr,
        patch_request: PatchRequest,
        attributes: Optional[StrictStr] = None,
        excluded_attributes: Optional[StrictStr] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """patch_users

        Patch a portion of the backing store

        :param id: (required)
        :type id: str
        :param integration: (required)
        :type integration: str
        :param tenant: (required)
        :type tenant: str
        :param patch_request: (required)
        :type patch_request: PatchRequest
        :param attributes:
        :type attributes: str
        :param excluded_attributes:
        :type excluded_attributes: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._patch_users_serialize(
            id=id,
            integration=integration,
            tenant=tenant,
            patch_request=patch_request,
            attributes=attributes,
            excluded_attributes=excluded_attributes,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
            '400': None,
            '404': None,
            '500': None,
            '501': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _patch_users_serialize(
        self,
        id,
        integration,
        tenant,
        patch_request,
        attributes,
        excluded_attributes,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if id is not None:
            _path_params['id'] = id
        if integration is not None:
            _path_params['integration'] = integration
        if tenant is not None:
            _path_params['tenant'] = tenant
        # process the query parameters
        if attributes is not None:
            
            _query_params.append(('attributes', attributes))
            
        if excluded_attributes is not None:
            
            _query_params.append(('excludedAttributes', excluded_attributes))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if patch_request is not None:
            _body_params = patch_request



        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/scim+json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
        ]

        return self.api_client.param_serialize(
            method='PATCH',
            resource_path='/api/v1/{tenant}/integrations/{integration}/scim/v2/Users/{id}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def query_users(
        self,
        integration: StrictStr,
        tenant: StrictStr,
        attributes: Optional[StrictStr] = None,
        excluded_attributes: Optional[StrictStr] = None,
        filter: Optional[StrictStr] = None,
        sort_by: Optional[StrictStr] = None,
        sort_order: Optional[Any] = None,
        start_index: Optional[StrictInt] = None,
        count: Optional[StrictInt] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ScimResource:
        """query_users

        Find by a combination of query parameters

        :param integration: (required)
        :type integration: str
        :param tenant: (required)
        :type tenant: str
        :param attributes:
        :type attributes: str
        :param excluded_attributes:
        :type excluded_attributes: str
        :param filter:
        :type filter: str
        :param sort_by:
        :type sort_by: str
        :param sort_order:
        :type sort_order: SortOrder
        :param start_index:
        :type start_index: int
        :param count:
        :type count: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._query_users_serialize(
            integration=integration,
            tenant=tenant,
            attributes=attributes,
            excluded_attributes=excluded_attributes,
            filter=filter,
            sort_by=sort_by,
            sort_order=sort_order,
            start_index=start_index,
            count=count,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '400': None,
            '404': None,
            '500': None,
            '501': None,
            '200': "ScimResource",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def query_users_with_http_info(
        self,
        integration: StrictStr,
        tenant: StrictStr,
        attributes: Optional[StrictStr] = None,
        excluded_attributes: Optional[StrictStr] = None,
        filter: Optional[StrictStr] = None,
        sort_by: Optional[StrictStr] = None,
        sort_order: Optional[Any] = None,
        start_index: Optional[StrictInt] = None,
        count: Optional[StrictInt] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[ScimResource]:
        """query_users

        Find by a combination of query parameters

        :param integration: (required)
        :type integration: str
        :param tenant: (required)
        :type tenant: str
        :param attributes:
        :type attributes: str
        :param excluded_attributes:
        :type excluded_attributes: str
        :param filter:
        :type filter: str
        :param sort_by:
        :type sort_by: str
        :param sort_order:
        :type sort_order: SortOrder
        :param start_index:
        :type start_index: int
        :param count:
        :type count: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._query_users_serialize(
            integration=integration,
            tenant=tenant,
            attributes=attributes,
            excluded_attributes=excluded_attributes,
            filter=filter,
            sort_by=sort_by,
            sort_order=sort_order,
            start_index=start_index,
            count=count,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '400': None,
            '404': None,
            '500': None,
            '501': None,
            '200': "ScimResource",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def query_users_without_preload_content(
        self,
        integration: StrictStr,
        tenant: StrictStr,
        attributes: Optional[StrictStr] = None,
        excluded_attributes: Optional[StrictStr] = None,
        filter: Optional[StrictStr] = None,
        sort_by: Optional[StrictStr] = None,
        sort_order: Optional[Any] = None,
        start_index: Optional[StrictInt] = None,
        count: Optional[StrictInt] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """query_users

        Find by a combination of query parameters

        :param integration: (required)
        :type integration: str
        :param tenant: (required)
        :type tenant: str
        :param attributes:
        :type attributes: str
        :param excluded_attributes:
        :type excluded_attributes: str
        :param filter:
        :type filter: str
        :param sort_by:
        :type sort_by: str
        :param sort_order:
        :type sort_order: SortOrder
        :param start_index:
        :type start_index: int
        :param count:
        :type count: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._query_users_serialize(
            integration=integration,
            tenant=tenant,
            attributes=attributes,
            excluded_attributes=excluded_attributes,
            filter=filter,
            sort_by=sort_by,
            sort_order=sort_order,
            start_index=start_index,
            count=count,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '400': None,
            '404': None,
            '500': None,
            '501': None,
            '200': "ScimResource",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _query_users_serialize(
        self,
        integration,
        tenant,
        attributes,
        excluded_attributes,
        filter,
        sort_by,
        sort_order,
        start_index,
        count,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if integration is not None:
            _path_params['integration'] = integration
        if tenant is not None:
            _path_params['tenant'] = tenant
        # process the query parameters
        if attributes is not None:
            
            _query_params.append(('attributes', attributes))
            
        if excluded_attributes is not None:
            
            _query_params.append(('excludedAttributes', excluded_attributes))
            
        if filter is not None:
            
            _query_params.append(('filter', filter))
            
        if sort_by is not None:
            
            _query_params.append(('sortBy', sort_by))
            
        if sort_order is not None:
            
            _query_params.append(('sortOrder', sort_order.value))
            
        if start_index is not None:
            
            _query_params.append(('startIndex', start_index))
            
        if count is not None:
            
            _query_params.append(('count', count))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/scim+json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/api/v1/{tenant}/integrations/{integration}/scim/v2/Users',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def update_users(
        self,
        id: StrictStr,
        integration: StrictStr,
        tenant: StrictStr,
        scim_resource: ScimResource,
        attributes: Optional[StrictStr] = None,
        excluded_attributes: Optional[StrictStr] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ScimResource:
        """update_users

        Update

        :param id: (required)
        :type id: str
        :param integration: (required)
        :type integration: str
        :param tenant: (required)
        :type tenant: str
        :param scim_resource: (required)
        :type scim_resource: ScimResource
        :param attributes:
        :type attributes: str
        :param excluded_attributes:
        :type excluded_attributes: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._update_users_serialize(
            id=id,
            integration=integration,
            tenant=tenant,
            scim_resource=scim_resource,
            attributes=attributes,
            excluded_attributes=excluded_attributes,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '400': None,
            '500': None,
            '501': None,
            '200': "ScimResource",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def update_users_with_http_info(
        self,
        id: StrictStr,
        integration: StrictStr,
        tenant: StrictStr,
        scim_resource: ScimResource,
        attributes: Optional[StrictStr] = None,
        excluded_attributes: Optional[StrictStr] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[ScimResource]:
        """update_users

        Update

        :param id: (required)
        :type id: str
        :param integration: (required)
        :type integration: str
        :param tenant: (required)
        :type tenant: str
        :param scim_resource: (required)
        :type scim_resource: ScimResource
        :param attributes:
        :type attributes: str
        :param excluded_attributes:
        :type excluded_attributes: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._update_users_serialize(
            id=id,
            integration=integration,
            tenant=tenant,
            scim_resource=scim_resource,
            attributes=attributes,
            excluded_attributes=excluded_attributes,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '400': None,
            '500': None,
            '501': None,
            '200': "ScimResource",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def update_users_without_preload_content(
        self,
        id: StrictStr,
        integration: StrictStr,
        tenant: StrictStr,
        scim_resource: ScimResource,
        attributes: Optional[StrictStr] = None,
        excluded_attributes: Optional[StrictStr] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """update_users

        Update

        :param id: (required)
        :type id: str
        :param integration: (required)
        :type integration: str
        :param tenant: (required)
        :type tenant: str
        :param scim_resource: (required)
        :type scim_resource: ScimResource
        :param attributes:
        :type attributes: str
        :param excluded_attributes:
        :type excluded_attributes: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._update_users_serialize(
            id=id,
            integration=integration,
            tenant=tenant,
            scim_resource=scim_resource,
            attributes=attributes,
            excluded_attributes=excluded_attributes,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '400': None,
            '500': None,
            '501': None,
            '200': "ScimResource",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _update_users_serialize(
        self,
        id,
        integration,
        tenant,
        scim_resource,
        attributes,
        excluded_attributes,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if id is not None:
            _path_params['id'] = id
        if integration is not None:
            _path_params['integration'] = integration
        if tenant is not None:
            _path_params['tenant'] = tenant
        # process the query parameters
        if attributes is not None:
            
            _query_params.append(('attributes', attributes))
            
        if excluded_attributes is not None:
            
            _query_params.append(('excludedAttributes', excluded_attributes))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if scim_resource is not None:
            _body_params = scim_resource


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/scim+json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/scim+json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
        ]

        return self.api_client.param_serialize(
            method='PUT',
            resource_path='/api/v1/{tenant}/integrations/{integration}/scim/v2/Users/{id}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )





# coding: utf-8

"""
    Kestra EE

    All API operations, except for Superadmin-only endpoints, require a tenant identifier in the HTTP path.<br/> Endpoints designated as Superadmin-only are not tenant-scoped.

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from kestrapy.models.condition import Condition
from kestrapy.models.level import Level
from kestrapy.models.state_type import StateType
from kestrapy.models.the_labels_to_pass_to_the_execution_created import TheLabelsToPassToTheExecutionCreated
from kestrapy.models.worker_group import WorkerGroup
from typing import Optional, Set
from typing_extensions import Self

class AbstractTrigger(BaseModel):
    """
    AbstractTrigger
    """ # noqa: E501
    min_log_level: Optional[Level] = Field(default=None, alias="minLogLevel")
    id: Annotated[str, Field(min_length=1, strict=True)]
    type: Annotated[str, Field(min_length=1, strict=True)]
    version: Optional[Annotated[str, Field(strict=True)]] = None
    description: Optional[StrictStr] = None
    conditions: Optional[List[Condition]] = None
    disabled: StrictBool
    worker_group: Optional[WorkerGroup] = Field(default=None, alias="workerGroup")
    log_level: Optional[Level] = Field(default=None, alias="logLevel")
    labels: Optional[TheLabelsToPassToTheExecutionCreated] = None
    stop_after: Optional[List[StateType]] = Field(default=None, alias="stopAfter")
    log_to_file: Optional[StrictBool] = Field(default=None, alias="logToFile")
    fail_on_trigger_error: Optional[StrictBool] = Field(default=None, alias="failOnTriggerError")
    additional_properties: Dict[str, Any] = {}
    __properties: ClassVar[List[str]] = ["minLogLevel", "id", "type", "version", "description", "conditions", "disabled", "workerGroup", "logLevel", "labels", "stopAfter", "logToFile", "failOnTriggerError"]

    @field_validator('id')
    def id_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if not re.match(r"^[a-zA-Z0-9][a-zA-Z0-9_-]*", value):
            raise ValueError(r"must validate the regular expression /^[a-zA-Z0-9][a-zA-Z0-9_-]*/")
        return value

    @field_validator('type')
    def type_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if not re.match(r"^[A-Za-z_$][A-Za-z0-9_$]*(\.[A-Za-z_$][A-Za-z0-9_$]*)*$", value):
            raise ValueError(r"must validate the regular expression /^[A-Za-z_$][A-Za-z0-9_$]*(\.[A-Za-z_$][A-Za-z0-9_$]*)*$/")
        return value

    @field_validator('version')
    def version_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if value is None:
            return value

        if not re.match(r"\d+\.\d+\.\d+(-[a-zA-Z0-9-]+)?|([a-zA-Z0-9]+)", value):
            raise ValueError(r"must validate the regular expression /\d+\.\d+\.\d+(-[a-zA-Z0-9-]+)?|([a-zA-Z0-9]+)/")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of AbstractTrigger from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * Fields in `self.additional_properties` are added to the output dict.
        """
        excluded_fields: Set[str] = set([
            "additional_properties",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in conditions (list)
        _items = []
        if self.conditions:
            for _item_conditions in self.conditions:
                if _item_conditions:
                    _items.append(_item_conditions.to_dict())
            _dict['conditions'] = _items
        # override the default output from pydantic by calling `to_dict()` of worker_group
        if self.worker_group:
            _dict['workerGroup'] = self.worker_group.to_dict()
        # override the default output from pydantic by calling `to_dict()` of labels
        if self.labels:
            _dict['labels'] = self.labels.to_dict()
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of AbstractTrigger from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "minLogLevel": obj.get("minLogLevel"),
            "id": obj.get("id"),
            "type": obj.get("type"),
            "version": obj.get("version"),
            "description": obj.get("description"),
            "conditions": [Condition.from_dict(_item) for _item in obj["conditions"]] if obj.get("conditions") is not None else None,
            "disabled": obj.get("disabled") if obj.get("disabled") is not None else False,
            "workerGroup": WorkerGroup.from_dict(obj["workerGroup"]) if obj.get("workerGroup") is not None else None,
            "logLevel": obj.get("logLevel"),
            "labels": TheLabelsToPassToTheExecutionCreated.from_dict(obj["labels"]) if obj.get("labels") is not None else None,
            "stopAfter": obj.get("stopAfter"),
            "logToFile": obj.get("logToFile"),
            "failOnTriggerError": obj.get("failOnTriggerError")
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj





# coding: utf-8

"""
    Kestra EE

    All API operations, except for Superadmin-only endpoints, require a tenant identifier in the HTTP path.<br/> Endpoints designated as Superadmin-only are not tenant-scoped.

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from kestrapy.models.banner import Banner
from kestrapy.models.edition_provider_edition import EditionProviderEdition
from kestrapy.models.left_sidebar_configuration import LeftSidebarConfiguration
from kestrapy.models.misc_controller_environment import MiscControllerEnvironment
from kestrapy.models.misc_controller_plugin_id_and_version import MiscControllerPluginIdAndVersion
from kestrapy.models.misc_controller_preview import MiscControllerPreview
from kestrapy.models.misc_controller_tenant_configuration_info import MiscControllerTenantConfigurationInfo
from kestrapy.models.query_filter_resource_field import QueryFilterResourceField
from kestrapy.models.right_sidebar_configuration import RightSidebarConfiguration
from typing import Optional, Set
from typing_extensions import Self

class MiscControllerEEConfiguration(BaseModel):
    """
    MiscControllerEEConfiguration
    """ # noqa: E501
    uuid: Optional[StrictStr] = None
    version: Optional[StrictStr] = None
    edition: Optional[EditionProviderEdition] = None
    commit_id: Optional[StrictStr] = Field(default=None, alias="commitId")
    commit_date: Optional[datetime] = Field(default=None, alias="commitDate")
    is_custom_dashboards_enabled: Optional[StrictBool] = Field(default=None, alias="isCustomDashboardsEnabled")
    is_task_run_enabled: Optional[StrictBool] = Field(default=None, alias="isTaskRunEnabled")
    is_anonymous_usage_enabled: Optional[StrictBool] = Field(default=None, alias="isAnonymousUsageEnabled")
    is_ui_anonymous_usage_enabled: Optional[StrictBool] = Field(default=None, alias="isUiAnonymousUsageEnabled")
    is_template_enabled: Optional[StrictBool] = Field(default=None, alias="isTemplateEnabled")
    environment: Optional[MiscControllerEnvironment] = None
    url: Optional[StrictStr] = None
    preview: Optional[MiscControllerPreview] = None
    system_namespace: Optional[StrictStr] = Field(default=None, alias="systemNamespace")
    hidden_labels_prefixes: Optional[List[StrictStr]] = Field(default=None, alias="hiddenLabelsPrefixes")
    resource_to_filters: Optional[List[QueryFilterResourceField]] = Field(default=None, alias="resourceToFilters")
    is_ai_enabled: Optional[StrictBool] = Field(default=None, alias="isAiEnabled")
    is_basic_auth_initialized: Optional[StrictBool] = Field(default=None, alias="isBasicAuthInitialized")
    plugins_hash: Optional[StrictInt] = Field(default=None, alias="pluginsHash")
    tenants: Optional[MiscControllerTenantConfigurationInfo] = None
    secrets_enabled: Optional[StrictBool] = Field(default=None, alias="secretsEnabled")
    supported_storages: Optional[List[MiscControllerPluginIdAndVersion]] = Field(default=None, alias="supportedStorages")
    supported_secrets: Optional[List[MiscControllerPluginIdAndVersion]] = Field(default=None, alias="supportedSecrets")
    plugin_management_enabled: Optional[StrictBool] = Field(default=None, alias="pluginManagementEnabled")
    plugin_custom_enabled: Optional[StrictBool] = Field(default=None, alias="pluginCustomEnabled")
    banner: Optional[Banner] = None
    mail_service_enabled: Optional[StrictBool] = Field(default=None, alias="mailServiceEnabled")
    outputs_in_internal_storage_enabled: Optional[StrictBool] = Field(default=None, alias="outputsInInternalStorageEnabled")
    left_sidebar: Optional[LeftSidebarConfiguration] = Field(default=None, alias="leftSidebar")
    right_sidebar: Optional[RightSidebarConfiguration] = Field(default=None, alias="rightSidebar")
    in_maintenance: Optional[StrictBool] = Field(default=None, alias="inMaintenance")
    password_regexp: Optional[StrictStr] = Field(default=None, alias="passwordRegexp")
    passwordless_enabled: Optional[StrictBool] = Field(default=None, alias="passwordlessEnabled")
    additional_properties: Dict[str, Any] = {}
    __properties: ClassVar[List[str]] = ["uuid", "version", "edition", "commitId", "commitDate", "isCustomDashboardsEnabled", "isTaskRunEnabled", "isAnonymousUsageEnabled", "isUiAnonymousUsageEnabled", "isTemplateEnabled", "environment", "url", "preview", "systemNamespace", "hiddenLabelsPrefixes", "resourceToFilters", "isAiEnabled", "isBasicAuthInitialized", "pluginsHash", "tenants", "secretsEnabled", "supportedStorages", "supportedSecrets", "pluginManagementEnabled", "pluginCustomEnabled", "banner", "mailServiceEnabled", "outputsInInternalStorageEnabled", "leftSidebar", "rightSidebar", "inMaintenance", "passwordRegexp", "passwordlessEnabled"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of MiscControllerEEConfiguration from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * Fields in `self.additional_properties` are added to the output dict.
        """
        excluded_fields: Set[str] = set([
            "additional_properties",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of environment
        if self.environment:
            _dict['environment'] = self.environment.to_dict()
        # override the default output from pydantic by calling `to_dict()` of preview
        if self.preview:
            _dict['preview'] = self.preview.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in resource_to_filters (list)
        _items = []
        if self.resource_to_filters:
            for _item_resource_to_filters in self.resource_to_filters:
                if _item_resource_to_filters:
                    _items.append(_item_resource_to_filters.to_dict())
            _dict['resourceToFilters'] = _items
        # override the default output from pydantic by calling `to_dict()` of tenants
        if self.tenants:
            _dict['tenants'] = self.tenants.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in supported_storages (list)
        _items = []
        if self.supported_storages:
            for _item_supported_storages in self.supported_storages:
                if _item_supported_storages:
                    _items.append(_item_supported_storages.to_dict())
            _dict['supportedStorages'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in supported_secrets (list)
        _items = []
        if self.supported_secrets:
            for _item_supported_secrets in self.supported_secrets:
                if _item_supported_secrets:
                    _items.append(_item_supported_secrets.to_dict())
            _dict['supportedSecrets'] = _items
        # override the default output from pydantic by calling `to_dict()` of banner
        if self.banner:
            _dict['banner'] = self.banner.to_dict()
        # override the default output from pydantic by calling `to_dict()` of left_sidebar
        if self.left_sidebar:
            _dict['leftSidebar'] = self.left_sidebar.to_dict()
        # override the default output from pydantic by calling `to_dict()` of right_sidebar
        if self.right_sidebar:
            _dict['rightSidebar'] = self.right_sidebar.to_dict()
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of MiscControllerEEConfiguration from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "uuid": obj.get("uuid"),
            "version": obj.get("version"),
            "edition": obj.get("edition"),
            "commitId": obj.get("commitId"),
            "commitDate": obj.get("commitDate"),
            "isCustomDashboardsEnabled": obj.get("isCustomDashboardsEnabled"),
            "isTaskRunEnabled": obj.get("isTaskRunEnabled"),
            "isAnonymousUsageEnabled": obj.get("isAnonymousUsageEnabled"),
            "isUiAnonymousUsageEnabled": obj.get("isUiAnonymousUsageEnabled"),
            "isTemplateEnabled": obj.get("isTemplateEnabled"),
            "environment": MiscControllerEnvironment.from_dict(obj["environment"]) if obj.get("environment") is not None else None,
            "url": obj.get("url"),
            "preview": MiscControllerPreview.from_dict(obj["preview"]) if obj.get("preview") is not None else None,
            "systemNamespace": obj.get("systemNamespace"),
            "hiddenLabelsPrefixes": obj.get("hiddenLabelsPrefixes"),
            "resourceToFilters": [QueryFilterResourceField.from_dict(_item) for _item in obj["resourceToFilters"]] if obj.get("resourceToFilters") is not None else None,
            "isAiEnabled": obj.get("isAiEnabled"),
            "isBasicAuthInitialized": obj.get("isBasicAuthInitialized"),
            "pluginsHash": obj.get("pluginsHash"),
            "tenants": MiscControllerTenantConfigurationInfo.from_dict(obj["tenants"]) if obj.get("tenants") is not None else None,
            "secretsEnabled": obj.get("secretsEnabled"),
            "supportedStorages": [MiscControllerPluginIdAndVersion.from_dict(_item) for _item in obj["supportedStorages"]] if obj.get("supportedStorages") is not None else None,
            "supportedSecrets": [MiscControllerPluginIdAndVersion.from_dict(_item) for _item in obj["supportedSecrets"]] if obj.get("supportedSecrets") is not None else None,
            "pluginManagementEnabled": obj.get("pluginManagementEnabled"),
            "pluginCustomEnabled": obj.get("pluginCustomEnabled"),
            "banner": Banner.from_dict(obj["banner"]) if obj.get("banner") is not None else None,
            "mailServiceEnabled": obj.get("mailServiceEnabled"),
            "outputsInInternalStorageEnabled": obj.get("outputsInInternalStorageEnabled"),
            "leftSidebar": LeftSidebarConfiguration.from_dict(obj["leftSidebar"]) if obj.get("leftSidebar") is not None else None,
            "rightSidebar": RightSidebarConfiguration.from_dict(obj["rightSidebar"]) if obj.get("rightSidebar") is not None else None,
            "inMaintenance": obj.get("inMaintenance"),
            "passwordRegexp": obj.get("passwordRegexp"),
            "passwordlessEnabled": obj.get("passwordlessEnabled")
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj





# coding: utf-8

"""
    Kestra EE

    All API operations, except for Superadmin-only endpoints, require a tenant identifier in the HTTP path.<br/> Endpoints designated as Superadmin-only are not tenant-scoped.

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from kestrapy.models.breakpoint import Breakpoint
from kestrapy.models.execution_kind import ExecutionKind
from kestrapy.models.execution_metadata import ExecutionMetadata
from kestrapy.models.execution_trigger import ExecutionTrigger
from kestrapy.models.label import Label
from kestrapy.models.state import State
from kestrapy.models.task_fixture import TaskFixture
from kestrapy.models.task_run import TaskRun
from typing import Optional, Set
from typing_extensions import Self

class Execution(BaseModel):
    """
    Execution
    """ # noqa: E501
    labels: Optional[List[Label]] = None
    id: StrictStr
    namespace: StrictStr
    flow_id: StrictStr = Field(alias="flowId")
    flow_revision: StrictInt = Field(alias="flowRevision")
    task_run_list: Optional[List[TaskRun]] = Field(default=None, alias="taskRunList")
    inputs: Optional[Dict[str, Dict[str, Any]]] = None
    outputs: Optional[Dict[str, Dict[str, Any]]] = None
    variables: Optional[Dict[str, Dict[str, Any]]] = None
    state: State
    parent_id: Optional[StrictStr] = Field(default=None, alias="parentId")
    original_id: Optional[StrictStr] = Field(default=None, alias="originalId")
    trigger: Optional[ExecutionTrigger] = None
    deleted: StrictBool
    metadata: Optional[ExecutionMetadata] = None
    schedule_date: Optional[datetime] = Field(default=None, alias="scheduleDate")
    trace_parent: Optional[StrictStr] = Field(default=None, alias="traceParent")
    fixtures: Optional[List[TaskFixture]] = None
    kind: Optional[ExecutionKind] = None
    breakpoints: Optional[List[Breakpoint]] = None
    additional_properties: Dict[str, Any] = {}
    __properties: ClassVar[List[str]] = ["labels", "id", "namespace", "flowId", "flowRevision", "taskRunList", "inputs", "outputs", "variables", "state", "parentId", "originalId", "trigger", "deleted", "metadata", "scheduleDate", "traceParent", "fixtures", "kind", "breakpoints"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of Execution from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * Fields in `self.additional_properties` are added to the output dict.
        """
        excluded_fields: Set[str] = set([
            "additional_properties",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in labels (list)
        _items = []
        if self.labels:
            for _item_labels in self.labels:
                if _item_labels:
                    _items.append(_item_labels.to_dict())
            _dict['labels'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in task_run_list (list)
        _items = []
        if self.task_run_list:
            for _item_task_run_list in self.task_run_list:
                if _item_task_run_list:
                    _items.append(_item_task_run_list.to_dict())
            _dict['taskRunList'] = _items
        # override the default output from pydantic by calling `to_dict()` of state
        if self.state:
            _dict['state'] = self.state.to_dict()
        # override the default output from pydantic by calling `to_dict()` of trigger
        if self.trigger:
            _dict['trigger'] = self.trigger.to_dict()
        # override the default output from pydantic by calling `to_dict()` of metadata
        if self.metadata:
            _dict['metadata'] = self.metadata.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in fixtures (list)
        _items = []
        if self.fixtures:
            for _item_fixtures in self.fixtures:
                if _item_fixtures:
                    _items.append(_item_fixtures.to_dict())
            _dict['fixtures'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in breakpoints (list)
        _items = []
        if self.breakpoints:
            for _item_breakpoints in self.breakpoints:
                if _item_breakpoints:
                    _items.append(_item_breakpoints.to_dict())
            _dict['breakpoints'] = _items
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        # set to None if schedule_date (nullable) is None
        # and model_fields_set contains the field
        if self.schedule_date is None and "schedule_date" in self.model_fields_set:
            _dict['scheduleDate'] = None

        # set to None if fixtures (nullable) is None
        # and model_fields_set contains the field
        if self.fixtures is None and "fixtures" in self.model_fields_set:
            _dict['fixtures'] = None

        # set to None if kind (nullable) is None
        # and model_fields_set contains the field
        if self.kind is None and "kind" in self.model_fields_set:
            _dict['kind'] = None

        # set to None if breakpoints (nullable) is None
        # and model_fields_set contains the field
        if self.breakpoints is None and "breakpoints" in self.model_fields_set:
            _dict['breakpoints'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Execution from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "labels": [Label.from_dict(_item) for _item in obj["labels"]] if obj.get("labels") is not None else None,
            "id": obj.get("id"),
            "namespace": obj.get("namespace"),
            "flowId": obj.get("flowId"),
            "flowRevision": obj.get("flowRevision"),
            "taskRunList": [TaskRun.from_dict(_item) for _item in obj["taskRunList"]] if obj.get("taskRunList") is not None else None,
            "inputs": obj.get("inputs"),
            "outputs": obj.get("outputs"),
            "variables": obj.get("variables"),
            "state": State.from_dict(obj["state"]) if obj.get("state") is not None else None,
            "parentId": obj.get("parentId"),
            "originalId": obj.get("originalId"),
            "trigger": ExecutionTrigger.from_dict(obj["trigger"]) if obj.get("trigger") is not None else None,
            "deleted": obj.get("deleted"),
            "metadata": ExecutionMetadata.from_dict(obj["metadata"]) if obj.get("metadata") is not None else None,
            "scheduleDate": obj.get("scheduleDate"),
            "traceParent": obj.get("traceParent"),
            "fixtures": [TaskFixture.from_dict(_item) for _item in obj["fixtures"]] if obj.get("fixtures") is not None else None,
            "kind": obj.get("kind"),
            "breakpoints": [Breakpoint.from_dict(_item) for _item in obj["breakpoints"]] if obj.get("breakpoints") is not None else None
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj


